#ifndef _LZW_PRIVATE_H_
#define _LZW_PRIVATE_H_

/* 
 * DO NOT TRY TO ACCESS ANY FIELDS OF THE FOLLOWING STRUCTURES!
 * ALL OF THEM ARE CONSIDERED PRIVATE ONLY AND THEIR DETAILS
 * MUST NOT LEAK TO THE USER CODE.
 */


#define LZWMAXCODE	4095	/* last code or dictionary index */
#define LZWMINCODE	256	/* first code or dictionary index */

#define LZWHASHSZ	6247	/* size of encoder hash table, must be prime */
#define LZWMAXBUF	4096	/* max buffer size to store input */

#define LZWNCTLCODES	8	/* number of reserved codes */

/* Dictionary table offset, excludes single-byte sequences and control codes. */
#define LZWTBLOFFS	(LZWMINCODE+LZWNCTLCODES-1)
/* Dictionary table size. */
#define LZWTBLSIZE	((LZWMAXCODE+1)-LZWTBLOFFS)

/* This is an encoder dictionary entry. */
struct lzwentry {
	short int	link;	/* index to next entry in a hash-bucket chain */
	short int	prefix;	/* index of the prefix sequence */
	unsigned char	chr;	/* additional character */
};

/* This is a decoder dictionary entry. */
struct lzwseq {
	short int	prefix;	/* index of the prefix sequence */
	unsigned char 	chr;	/* additional character */
};

/* LZW encoder context. */
struct lzw_encoder {
	int		bitlen;		 /* length of the code (in bits) */
	int		nextcode;	 /* dictionary index of the next code */
	int		left;		 /* number of less significant bits of the code in `c' */
	unsigned char	c;		 /* less significant bits of the code, partially filled byte */
	short int	hash[LZWHASHSZ]; /* dictionary hash, indicies to dic[] */
	unsigned char	cbuf[8];	 /* output circle buffer, where the code bits are collected */
	int		ctail, chead;	 /* circle buffer read/write indicies */
	struct lzwentry dic[LZWTBLSIZE]; /* dictionary table indexed by the (code-LZWTBLOFFS) */
	int		prevcode;	 /* previous sequence, single char (-1) or dic[prevcode-LZWTBLOFFS] (>=0) */
};

/* LZW decoder context. */
struct lzw_decoder {
	int		bitlen;		     /* length of the code (in bits) */
	int		nextcode;	     /* dictionary index of the next code */
	int		left;		     /* number of code bits left to extract */
	int		code;		     /* extracted bits of the code */
	unsigned char	prev[LZWMAXBUF];     /* previously decoded sequence */
	int		prevlen;	     /* length of the previously decoded sequence */
	struct lzwseq	dic[LZWTBLSIZE];
	int		prevcode;
};


#endif /* _LZW_PRIVATE_H_ */

